VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ImageProcessing"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' +-------------------------------- DISCLAIMER ---------------------------------+
' |                                                                             |
' | This application program is provided to you free of charge as an example.   |
' | Despite the considerable efforts of Euresys personnel to create a usable    |
' | example, you should not assume that this program is error-free or suitable  |
' | for any purpose whatsoever.                                                 |
' |                                                                             |
' | EURESYS does not give any representation, warranty or undertaking that this |
' | program is free of any defect or error or suitable for any purpose. EURESYS |
' | shall not be liable, in contract, in torts or otherwise, for any damages,   |
' | loss, costs, expenses or other claims for compensation, including those     |
' | asserted by third parties, arising out of or in connection with the use of  |
' | this program.                                                               |
' |                                                                             |
' +-----------------------------------------------------------------------------+

Option Explicit

Option Base 0

'Source and temporary images
Private m_SrcImg As New EImageBW8
Private m_ImgProcess As New EImageBW8


'Source and temporary masks
Private m_MaskImg As New EImageBW8
Private m_MaskProcess As New EImageBW8


'ROI to analyze one test tube at a time
Private m_SrcROI As New EROIBW8
Private m_MaskROI As New EROIBW8

'EasyGauge tools to register the mask
Private m_WorldShape As New EWorldShape
Private m_LineGauge As ELineGauge
Private m_PointGauge As EPointGauge

'EasyObject tools to detect reactions
Private m_CodedImage(4) As New ECodedImage2
Private m_Encoder As New EImageEncoder
Private m_Selection(4) As New EObjectSelection


'Canny Edge detector tool to detect bubbles
Private m_CannyEdge As New ECannyEdgeDetector

'Reference offset and rotation values
Private m_ReferenceAngle As Single
Private m_ReferencePositionX As Single
Private m_ReferencePositionY As Single

Private m_EasyImage As New EasyImage

Public Sub Initialize()


    On Error GoTo Failure

    Dim sPath As String
    
    sPath = Left(App.Path, InStrRev(App.Path, "\") - 1)
    
    'Load the EasyGauge model file
    m_WorldShape.Load sPath & "\Resources\Model\ClinicalAnalysis.GGE", True
    
    Set m_LineGauge = m_WorldShape.GetDaughter(0)
    Set m_PointGauge = m_LineGauge.GetDaughter(0)
    
    ' Load the reference image
    ' This reference image was used to create the flexible mask
    m_SrcImg.Load sPath & "\Resources\Images\ClinicalAnalysis_01.bmp"
    
    ' Evaluate the reference image offset and rotation
    m_WorldShape.Process m_SrcImg
    m_ReferenceAngle = m_LineGauge.MeasuredLine.angle
    m_ReferencePositionX = m_LineGauge.MeasuredLine.centerX + m_PointGauge.GetMeasuredPoint().X
    m_ReferencePositionY = m_LineGauge.MeasuredLine.centerY + m_PointGauge.GetMeasuredPoint().Y
    
    ' Load the corresponding flexible mask
    m_MaskImg.Load sPath & "\Resources\Mask\FlexMask.bmp"
    
    ' Resize the temporary mask image
    m_MaskProcess.SetSize m_MaskImg.Width, m_MaskImg.Height
    
    ' Attach an ROI to it
    m_MaskROI.Attach m_MaskProcess
    
    'Empty the source image object
    m_SrcImg.SetSize 0, 0
    
   On Error GoTo 0
    Exit Sub
    
Failure:
    ' Display the error description
    MsgBox "Error: " & Err.Description, vbOKOnly, "ClinicalAnalysis"
            
End Sub

Public Sub Inspect(Filename As String)


    On Error GoTo Failure
    
    Dim I As Integer
    
    ' Temporary image for preprocessing
    Dim m_TempImg As New EImageBW8
    
    'Load the image for inspection
    m_SrcImg.Load Filename

    'Resize the temporary image accordingly
    m_ImgProcess.SetSize m_SrcImg.Width, m_SrcImg.Height
    
    
    'Attach an ROI to it
    m_SrcROI.Attach m_ImgProcess
    
    'Evaluate the offset and the rotation for the current image
    m_WorldShape.Process m_SrcImg, True
    Dim m_Angle As Single
    Dim m_X1 As Single
    Dim m_Y1 As Single
    
    m_Angle = m_LineGauge.MeasuredLine.angle - m_ReferenceAngle
    m_X1 = m_LineGauge.MeasuredLine.centerX + m_PointGauge.GetMeasuredPoint().X
    m_Y1 = m_LineGauge.MeasuredLine.centerY + m_PointGauge.GetMeasuredPoint().Y
    
    'Register the mask according to the offset and the rotation of the current image
    m_EasyImage.ScaleRotate m_MaskImg, m_ReferencePositionX, m_ReferencePositionY, m_X1, m_Y1, 1#, 1#, -m_Angle, m_MaskProcess, 8
    
    
    'Draw the detected objects for each reagents
    For I = 0 To 3
    
        'Make a copy of the image for processing
        m_EasyImage.OperImage EArithmeticLogicOperation_Copy, m_SrcImg, m_ImgProcess
        
        'Position the image ROI
        m_SrcROI.SetPlacement m_X1 - 25 - (150 * (2 - I)), 2, 150, 308
        
        'Position the mask ROI
        m_MaskROI.SetPlacement m_X1 - 25 - (150 * (2 - I)), 2, 150, 308
        
        'Clear the objects selection
        m_Selection(I).Clear
        
        
        'Set the thresholding mode for the blob analysis and specify which layers to encode
        m_Encoder.GrayscaleSingleThresholdSegmenter.Mode = EGrayscaleSingleThreshold_Absolute
        m_Encoder.GrayscaleSingleThresholdSegmenter.BlackLayerEncoded = True
        m_Encoder.GrayscaleSingleThresholdSegmenter.WhiteLayerEncoded = False
        
        Select Case I
        
            Case 0
                'Preprocess the image for bubble detection
                m_TempImg.SetSize m_ImgProcess.Width, m_ImgProcess.Height
                m_EasyImage.WhiteTopHatDisk m_ImgProcess, m_TempImg, 5
             
                m_EasyImage.Median m_TempImg, m_ImgProcess
                
                'Canny edge for bubbles detection
                m_CannyEdge.HighThreshold = 0.2
                m_CannyEdge.LowThreshold = 0.1
                m_CannyEdge.ThresholdingMode = ECannyThresholdingMode_Relative
                m_CannyEdge.SmoothingScale = 1.1
                m_CannyEdge.Apply m_ImgProcess, m_ImgProcess
                
                'Set the absolute threshold value and specify which layers to encode
                m_Encoder.GrayscaleSingleThresholdSegmenter.AbsoluteThreshold = 90
                m_Encoder.GrayscaleSingleThresholdSegmenter.BlackLayerEncoded = False
                m_Encoder.GrayscaleSingleThresholdSegmenter.WhiteLayerEncoded = True
                
                'Encode the objects with masking
                m_Encoder.EncodeMasked m_SrcROI, m_MaskROI, m_CodedImage(I)
                m_Selection(I).AddObjects m_CodedImage(I)
                m_Selection(I).RemoveUsingUnsignedIntegerFeature EFeature_Area, 20, ESingleThresholdMode_LessEqual
            
            
            Case 1
                'Set the absolute threshold value
                m_Encoder.GrayscaleSingleThresholdSegmenter.AbsoluteThreshold = 80
                'Encode objects with masking
                m_Encoder.EncodeMasked m_SrcROI, m_MaskROI, m_CodedImage(I)
                m_Selection(I).AddObjects m_CodedImage(I)
                m_Selection(I).RemoveUsingUnsignedIntegerFeature EFeature_Area, 50, ESingleThresholdMode_LessEqual
            
            Case 2
                'Set the absolute threshold value
                m_Encoder.GrayscaleSingleThresholdSegmenter.AbsoluteThreshold = 80
                'Encode objects with masking
                m_Encoder.EncodeMasked m_SrcROI, m_MaskROI, m_CodedImage(I)
                m_Selection(I).AddObjects m_CodedImage(I)
            
            
            
            Case 3
                'Set the absolute threshold value
                m_Encoder.GrayscaleSingleThresholdSegmenter.AbsoluteThreshold = 50
                'Encode objects with masking
                m_Encoder.EncodeMasked m_SrcROI, m_MaskROI, m_CodedImage(I)
                m_Selection(I).AddObjects m_CodedImage(I)
                m_Selection(I).RemoveUsingUnsignedIntegerFeature EFeature_Area, 100, ESingleThresholdMode_LessEqual
            
        
        End Select
    
    Next I
    
    On Error GoTo 0
    Exit Sub
    
Failure:
    ' Display the error description
    MsgBox "Error: " & Err.Description, vbOKOnly, "ClinicalAnalysis"

End Sub

Public Sub Draw(HDCContext As Long)

    On Error GoTo Failure

    Dim I As Integer
    Dim m_RedColor As ERGBColor
    Dim m_BlueColor As ERGBColor
    
    'Set the color components
    m_RedColor.Red = 255
    m_BlueColor.Blue = 255


    'Draw the source image
    m_SrcImg.Draw HDCContext

    'Draw the detected objects for each reagents
    For I = 0 To 3


        If m_Selection(I).ElementCount > 0 Then
        
            'Draw the detected objects
            m_CodedImage(I).DrawSelectionWithColor HDCContext, m_RedColor, m_Selection(I)
                   
            'Draw the objects gravity center
            m_CodedImage(I).DrawSelectionElementsFeatureWithColor HDCContext, m_BlueColor, EDrawableFeature_GravityCenter, m_Selection(I)
            
        End If

    Next I

   On Error GoTo 0
    Exit Sub

Failure:
    ' Display the error description
    MsgBox "Error: " & Err.Description, vbOKOnly, "ClinicalAnalysis"

End Sub

